import { getPackageVersion } from "../../extension/package-version.js";
import { resolveExtensionConfiguration } from "../../extension_configuration/resolve.js";
import { detectSupportedSurfaces, logSurfaces, } from "../../extension_configuration/detect-supported-surfaces.js";
import path, { resolve } from "node:path";
import fs, { existsSync, mkdirSync, writeFileSync } from "node:fs";
import chalk from "chalk";
import { ZodError } from "zod";
export const MANIFEST_FILE_PATH = ".ntli/site/static/manifest.json";
/**
 * Creates a manifest.json file in the extension's build directory.
 */
export async function createManifest(extension, opts) {
    let config;
    try {
        config = await resolveExtensionConfiguration({ cwd: opts.cwd });
    }
    catch (error) {
        if (error instanceof Error && error?.cause instanceof ZodError) {
            const formattedErrors = error?.cause?.issues
                ?.map?.((issue) => chalk.red(`[${issue.path.join(".")}]: ${issue.message}`))
                .join(", ");
            throw new Error(`Invalid extension.yaml:\n${formattedErrors}`);
        }
        throw error;
    }
    let uiSurfaces = [];
    if (config.ui?.surfaces != undefined && config.ui.surfaces.length > 0) {
        // TODO(ndhoule): This is a fairly weak check. We should check to be sure the user has actually
        // emitted artifacts to the build directory defined at `netlify.toml#build.publish`. (We should
        // actually use @netlify/config)
        uiSurfaces = config.ui.surfaces;
        console.log(`\n${chalk.blue(`NetlifySDK`)} Extension UI surfaces (extension.yaml):`);
        logSurfaces(uiSurfaces);
        console.log(`\nℹ The SDK can automatically detect the surfaces if you remove the "config.ui.surfaces" field from your extension.yaml file.\n`);
    }
    if (!config.ui?.surfaces || config.ui.surfaces.length === 0) {
        uiSurfaces = detectSupportedSurfaces(opts.cwd);
        if (uiSurfaces.length > 0) {
            console.log(`\n${chalk.blue(`NetlifySDK`)} Detected Extension UI surfaces:`);
            logSurfaces(uiSurfaces);
            console.log(`\nℹ To override the detected surfaces, add the "config.ui.surfaces" field to your extension.yaml file.\n`);
        }
    }
    const version = getPackageVersion();
    const handlersToRun = extension.getBuildEventHandlersToRun({
        netlifyConfig: {},
    });
    const hasBuildEventHandler = Object.keys(handlersToRun).length > 0;
    const supports = extension.netlifyConnectPlugin
        ? {
            connect: extension.netlifyConnectPlugin.supports.connect,
            // TODO: "supports.create" in the public API was renamed to "supports.visualEditor"
            // but this manifest system still expects "supports.create"
            // We should update the manifest system to expect "supports.visualEditor"
            create: extension.netlifyConnectPlugin.supports.visualEditor,
        }
        : undefined;
    const manifest = {
        sdkVersion: version,
        hasDataIntegration: !!extension.netlifyConnectPlugin,
        hasBuildEventHandler,
        hasIntegrationUI: uiSurfaces.length > 0,
        uiSurfaces,
        supports,
    };
    if (!existsSync(resolve(opts.cwd, ".ntli/site/static"))) {
        mkdirSync(resolve(opts.cwd, ".ntli/site/static"), { recursive: true });
    }
    writeFileSync(resolve(opts.cwd, MANIFEST_FILE_PATH), JSON.stringify(manifest, null, 2));
    return manifest;
}
/**
 * Updates the manifest.json file in the extension's build directory.
 * Merges the provided data with the existing manifest.
 */
export function updateManifest(data, opts) {
    const manifest = getManifest(opts);
    const payload = {
        ...manifest,
        ...data,
    };
    if (manifest?.supports || data?.supports) {
        payload.supports = {
            ...(manifest.supports ?? {}),
            ...(data.supports ?? {}),
        };
    }
    if (!payload.uiSurfaces || payload.uiSurfaces.length === 0) {
        payload.hasIntegrationUI = false;
    }
    fs.writeFileSync(path.resolve(opts.cwd, MANIFEST_FILE_PATH), JSON.stringify(payload, null, 2));
}
/**
 * Reads the manifest.json file in the extension's build directory.
 * Returns the parsed manifest json object.
 */
export function getManifest(opts) {
    const manifestFilePath = path.resolve(opts.cwd, MANIFEST_FILE_PATH);
    const manifestFile = fs.readFileSync(manifestFilePath, "utf8");
    const manifest = JSON.parse(manifestFile);
    return manifest;
}
//# sourceMappingURL=manifest.js.map