import { ZodError } from "zod";
import chalk from "chalk";
import chokidar from "chokidar";
import { detectSupportedSurfaces, logSurfaces, } from "../../extension_configuration/detect-supported-surfaces.js";
import { getManifest, updateManifest } from "../build/manifest.js";
import { resolveExtensionConfiguration } from "../../extension_configuration/resolve.js";
/**
 * Watches for changes in the extension source files and updates the manifest.json accordingly.
 */
export function startDevWatcher(opts) {
    const watcher = chokidar.watch(opts.cwd, {
        ignored: [
            /node_modules/, // ignore node_modules
            /(^|[/\\])\../, // ignore hidden files (.git, .ntli, .netlify, etc)
        ],
        persistent: true,
    });
    /**
     * In Node ^20 EventEmitter is not a generic type so chokidar event emitter types are not populated correctly.
     * Since FSWatcher just extends EventEmitter we can cast it here.
     */
    watcher.on("all", (event) => {
        if (event === "change") {
            handleFileChange(opts);
        }
    });
    const handleExit = () => {
        watcher.close().then(() => {
            process.exit(0);
        });
    };
    process.on("SIGINT", handleExit); // Ctrl+C
    process.on("SIGTERM", handleExit); // Kill process
}
async function handleFileChange(opts) {
    let extensionConfig;
    try {
        extensionConfig = await resolveExtensionConfiguration({
            cwd: opts.cwd,
        });
    }
    catch (error) {
        if (error instanceof Error && error?.cause instanceof ZodError) {
            const formattedErrors = error?.cause?.issues
                ?.map?.((issue) => chalk.red(`[${issue.path.join(".")}]: ${issue.message}`))
                .join(", ");
            console.error(`Invalid extension.yaml:\n${formattedErrors}`);
            return;
        }
        console.error(error);
        return;
    }
    if (!extensionConfig) {
        return;
    }
    let manifest;
    try {
        manifest = getManifest({ cwd: opts.cwd });
    }
    catch (error) {
        console.error(`Failed to read manifest.json:`, error);
        return;
    }
    try {
        if (extensionConfig.ui?.surfaces != undefined &&
            extensionConfig.ui.surfaces.length > 0) {
            updateExtensionConfigSurfaces(manifest, extensionConfig, opts);
            return;
        }
        updateDetectedSurfaces(manifest, opts);
    }
    catch (e) {
        console.error(`Failed to update manifest.json:`, e);
    }
}
function updateExtensionConfigSurfaces(manifest, extensionConfig, opts) {
    if (!extensionConfig?.ui?.surfaces ||
        extensionConfig?.ui?.surfaces?.length === 0) {
        return;
    }
    if (manifest?.uiSurfaces) {
        // check if there are any differences
        const existingSurfaces = new Set(manifest.uiSurfaces);
        const newSurfaces = new Set(extensionConfig.ui.surfaces);
        if (existingSurfaces.size === newSurfaces.size &&
            // @ts-expect-error
            [...existingSurfaces].every((value) => newSurfaces.has(value))) {
            return; // return early if there are no differences in the surfaces
        }
    }
    console.log(`\n${chalk.blue(`NetlifySDK`)} Extension UI surfaces (extension.yaml):`);
    logSurfaces(extensionConfig.ui.surfaces, manifest?.uiSurfaces);
    updateManifest({
        hasIntegrationUI: true,
        uiSurfaces: extensionConfig.ui.surfaces,
    }, { cwd: opts.cwd });
}
function updateDetectedSurfaces(existingManifest, opts) {
    const uiSurfaces = detectSupportedSurfaces(opts.cwd);
    if (existingManifest?.uiSurfaces) {
        // check if there are any differences
        const existingSurfaces = new Set(existingManifest.uiSurfaces);
        const newSurfaces = new Set(uiSurfaces);
        if (existingSurfaces.size === newSurfaces.size &&
            // @ts-expect-error -- Argument of type 'string' is not assignable to parameter of type 'Surfaces'.ts(2345)
            [...existingSurfaces].every((value) => newSurfaces.has(value))) {
            return;
        }
    }
    console.log(`\n${chalk.blue(`NetlifySDK`)} Detected Extension UI surfaces:`);
    logSurfaces(uiSurfaces, existingManifest?.uiSurfaces);
    updateManifest({ uiSurfaces, hasIntegrationUI: uiSurfaces.length > 0 }, { cwd: opts.cwd });
}
//# sourceMappingURL=dev_watcher.js.map