import { Project, SyntaxKind } from "ts-morph";
import { Surfaces } from "@netlify/sdk/ui";
import chalk from "chalk";
import path from "node:path";
/**
 * Recursively check all import statements in the source file and its dependencies
 * to detect all surfaces used in the project.
 */
function checkImportFiles(file, sourceFiles, detectedSurfaces = []) {
    if (!file)
        return;
    const importDeclarations = file.getImportDeclarations();
    for (const importDeclaration of importDeclarations) {
        const sourceFile = importDeclaration.getModuleSpecifierSourceFile();
        const moduleSpecifierSourceFilePath = sourceFile?.getFilePath();
        if (!sourceFile ||
            importDeclaration.isTypeOnly() ||
            moduleSpecifierSourceFilePath?.includes("node_modules") ||
            !sourceFiles.find((sourceFile) => sourceFile.getFilePath() === moduleSpecifierSourceFilePath)) {
            // Skip type-only imports,
            // imports that are not part of the project source files,
            // and imports from node_modules
            continue;
        }
        const sourceFileImportDeclarations = sourceFile.getImportDeclarations();
        for (const sourceFileImportDeclaration of sourceFileImportDeclarations) {
            if (sourceFileImportDeclaration.isTypeOnly() ||
                sourceFileImportDeclaration.getModuleSpecifierValue() !==
                    "@netlify/sdk/ui/react") {
                // Skip type-only imports and imports that are not from @netlify/sdk/ui/react
                continue;
            }
            const bindings = sourceFileImportDeclaration
                .getImportClause()
                ?.getNamedBindings();
            const importBindings = bindings && bindings.isKind(SyntaxKind.NamedImports)
                ? bindings.getElements()?.filter((node) => !node.isTypeOnly())
                : undefined;
            const surfacesImportBinding = importBindings?.find?.((node) => {
                return (node.getName() === "Surfaces" && node.getSourceFile()?.getBaseName());
            });
            if (surfacesImportBinding) {
                const importBindingIdentifier = surfacesImportBinding.getNameNode();
                const referenceNodes = importBindingIdentifier.isKind(SyntaxKind.Identifier)
                    ? importBindingIdentifier.findReferencesAsNodes()
                    : undefined;
                if (referenceNodes) {
                    referenceNodes?.forEach((node) => {
                        const possibleSurfaceName = node
                            ?.getParent()
                            ?.getLastChildByKind(SyntaxKind.Identifier)
                            ?.getText();
                        // @ts-expect-error -- Argument of type 'string' is not assignable to parameter of type 'Surfaces'
                        if (possibleSurfaceName && Surfaces?.[possibleSurfaceName]) {
                            // @ts-expect-error -- Argument of type 'string' is not assignable to parameter of type 'Surfaces'
                            if (!detectedSurfaces.includes(Surfaces[possibleSurfaceName])) {
                                // @ts-expect-error -- Argument of type 'string' is not assignable to parameter of type 'Surfaces'
                                const surface = Surfaces[possibleSurfaceName];
                                detectedSurfaces.push(surface);
                            }
                        }
                    });
                }
            }
        }
        checkImportFiles(sourceFile, sourceFiles, detectedSurfaces);
    }
    return detectedSurfaces;
}
/**
 * Detects all surfaces used in the project source files by checking all import statements and their references.
 */
export const detectSupportedSurfaces = (cwd) => {
    const projectBaseDir = path.resolve(cwd, "src/ui");
    try {
        const supportedSurfaces = [];
        const project = new Project();
        project.addSourceFilesAtPaths(projectBaseDir + "/**/*.{ts,tsx}");
        const sourceFiles = project.getSourceFiles();
        const entryFile = project.getSourceFileOrThrow("index.tsx");
        checkImportFiles(entryFile, sourceFiles, supportedSurfaces);
        return supportedSurfaces;
    }
    catch {
        return [];
    }
};
/**
 * Logs the surfaces that are added, removed, or remain the same
 */
export function logSurfaces(surfaces, previousSurfaces) {
    if (!previousSurfaces) {
        console.log(surfaces.map((s) => `${chalk.green("•")} ${s}`).join("\n"));
        return;
    }
    const newSurfaces = surfaces.filter((surface) => !previousSurfaces?.includes(surface));
    const removedSurfaces = previousSurfaces?.filter((surface) => !surfaces?.includes?.(surface));
    const sameSurfaces = surfaces.filter((surface) => previousSurfaces?.includes(surface));
    if (sameSurfaces && sameSurfaces?.length > 0) {
        console.log(sameSurfaces.map((s) => `${chalk.yellow("•")} ${s}`).join("\n"));
    }
    if (newSurfaces && newSurfaces?.length > 0) {
        console.log(newSurfaces.map((s) => `${chalk.green("•")} ${s}`).join("\n"));
    }
    if (removedSurfaces && removedSurfaces.length > 0) {
        console.log(`${chalk.red(`Removed surfaces`)}\n${removedSurfaces.map((s) => `${chalk.red("-")} ${s}`).join("\n")}`);
    }
}
//# sourceMappingURL=detect-supported-surfaces.js.map